
#include "amrcodec.h"

#ifdef ETSI
	#error ETSI should not be defined.
#endif

void* amr_StartEncoder()
{
	// dtx=0; // No Discontinuous Transmission (DTX)
	void* enstate = Encoder_Interface_init(0);
	return enstate;
}

int amr_StopEncoder(void* enstate)
{
	Encoder_Interface_exit(enstate);
	return 0;
}

void* amr_StartDecoder()
{
	/* init decoder */
	void* destate = Decoder_Interface_init();
	return destate;
}

int amr_StopDecoder(void* destate)
{
	Decoder_Interface_exit(destate);
	return 0;
}

int amr_GetDataSize(int dec_mode)
{
	int read_size;

	switch (dec_mode){
      case 0:
         read_size = 12;
         break;
      case 1:
         read_size = 13;
         break;
      case 2:
         read_size = 15;
         break;
      case 3:
         read_size = 17;
         break;
      case 4:
         read_size = 18;
         break;
      case 5:
         read_size = 20;
         break;
      case 6:
         read_size = 25;
         break;
      case 7:
         read_size = 30;
         break;
      case 8:
         read_size = 5;
         break;
      case 15:
         read_size = 0;
      default:
         read_size = 0;
         break;
      };
	return read_size;
}

int amr_Encode(void* enstate, enum Mode req_mode, void *input, int inputSizeBytes, void *output, int *outputSizeBytes)
{
	char *dataIn = (char *)input;
	char *dataOut = (char *)output;
	int nFrames,i;
	unsigned char c;

	// MR475, MR515, MR59, MR67, MR74, MR795, MR102, MR122

	int byte_counter=0;

	if (inputSizeBytes > 0) {
		if (inputSizeBytes / 320 > *outputSizeBytes / amr_GetDataSize(req_mode)) // check output buffer len
		{
			*outputSizeBytes = 0;
			return -10;
		}
		else if (inputSizeBytes % 320 != 0) {	// input must be multiple of 10ms (160 bytes of pcm 16bit 8kHz Mono)
			*outputSizeBytes = 0;
			return -20;
		}
		else 
		{
/*			for ( i = 0 ; i < inputSizeBytes ; i+=2)
			{
				c = dataIn[i];
				dataIn[i] = dataIn[i+1];
				dataIn[i+1] = c;
			}
*/
			// convert each frame in input
			nFrames = inputSizeBytes / 320;
			*outputSizeBytes=0;
			for (i = 0; i < nFrames; i++) 
			{
				char * AMRData = (char *)&dataOut[*outputSizeBytes];
				char * pcmData = (char *)&dataIn[320 * i];

			     /* call encoder */
			    byte_counter = Encoder_Interface_Encode(enstate, req_mode, (short *)pcmData, (unsigned char *)AMRData, 0);
				*outputSizeBytes += byte_counter;
			}
		}
	}
	return 0;
}


enum Mode amr_Decode(void* destate, void *input, int inputSizeBytes, void *output, int *outputSizeBytes)
{
	char *dataIn = (char *)input;
	char *dataOut = (char *)output;
	int i,n;
	int dec_mode;
	int read_size;
	unsigned char c;

	n=0;i=0;
	while (n<inputSizeBytes)
	{
		char * pcmData = (char *)&dataOut[320 * i];
		char  * AMRData = (char  *)&dataIn[n];
		dec_mode = AMRData[0] & 0x000F;
		read_size=amr_GetDataSize(dec_mode);
 		n+=read_size+1;
		i++;
		if(*outputSizeBytes<i*320)
			break;
		/* call decoder */
		Decoder_Interface_Decode(destate, (unsigned char *)AMRData, (short*) pcmData, 0);
	}
	*outputSizeBytes = 320*i;
	
/*	for ( i = 0 ; i < *outputSizeBytes ; i+=2)
	{
		c = dataOut[i];
		dataOut[i] = dataOut[i+1];
		dataOut[i+1] = c;
	}
*/
	return (enum Mode) dec_mode;
}
